#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>

#include "../inc/lc6_config.h"
#include "../inc/lc6_crypto.h"

LC6_CTX* lc6config_load(char *path, unsigned char *password) {
	LC6_CTX *conf;

	conf = malloc(sizeof(LC6_CTX));
	assert(conf);
	memset(conf, '\0', sizeof(LC6_CTX));

	lc6config_load_file(conf, path, LC6_CONFIG_BOOTSTRAP, password);
	lc6config_load_file(conf, path, LC6_CONFIG_USER, password);
	lc6config_load_file(conf, path, LC6_CONFIG_FRIEND, password);

	// complete defaults

	// conf->path
	strncpy(conf->path, path, sizeof(conf->path));
	conf->path[sizeof(conf->path)-1]='\0';

	// conf->node
	// generate new keys at every startup.
	// those are only used for direct signaling to
	// the neighbours.
	conf->node = malloc(sizeof(LC6_USER));
	assert(conf->node);
	lc6crypto_genuserkey(conf->node);

	// conf->user may stay NULL

	// conf->friend maybe you're all alone, that's ok tho.

	// conf->bootstrap
	// if we can't bootstrap, we can't connect to the network
	// if ( !conf->bootstrap )
	//	return NULL;

	return conf;
}

void lc6config_load_file(
	LC6_CTX *conf,
	char *path,
	char *filename,
	unsigned char *password)
{
	unsigned char *data = NULL;
	int data_len = 0;
	char file[MAXPATHLEN];

	snprintf(file, sizeof(file), "%s/%s", path, filename);

	if ( lc6crypto_readfile(file, password, &data, &data_len) != 0 )
		return;

	if ( strcmp(filename, LC6_CONFIG_BOOTSTRAP) == 0 ) {
		LC6_BOOTSTRAP *prev = NULL;
		LC6_BOOTSTRAP *chain = (LC6_BOOTSTRAP*)data;
		int offset = 0;
		conf->bootstrap = (LC6_BOOTSTRAP*)data;

		while(offset<data_len/sizeof(LC6_BOOTSTRAP)) {
			LC6_BOOTSTRAP *node = malloc(sizeof(LC6_BOOTSTRAP));
			assert(node);
			memset(node, '\0', sizeof(LC6_BOOTSTRAP));
			memcpy(node, chain+offset, sizeof(LC6_BOOTSTRAP));

			if ( ! prev )
				conf->bootstrap = node;
			else
				prev->next = node;

			node->prev = prev;
			node->next = NULL;

			prev = node;
			offset++;
		}
	}
	else if ( strcmp(filename, LC6_CONFIG_USER) == 0 ) {
		LC6_USER *user = malloc(sizeof(LC6_USER));
		assert(user);
		memcpy(user, data, sizeof(LC6_USER));
		conf->user = user;
	}
	else if ( strcmp(filename, LC6_CONFIG_FRIEND) == 0 ) {
		LC6_FRIEND *prev = NULL;
		LC6_FRIEND *chain = (LC6_FRIEND*)data;
		int offset = 0;

		while(offset<data_len/sizeof(LC6_FRIEND)) {
			LC6_FRIEND *node = malloc(sizeof(LC6_FRIEND));

			assert(node);
			memset(node, '\0', sizeof(LC6_FRIEND));
			memcpy(node, chain+offset, sizeof(LC6_FRIEND));

			if ( ! prev )
				conf->friend = node;
			else
				prev->next = node;

			node->prev = prev;
			node->next = NULL;

			prev = node;
			offset++;
		}
	}

	free(data);
}

void lc6config_bootstrap_add(LC6_CTX *conf, LC6_BOOTSTRAP *bs) {
	assert(conf);
	assert(bs);

	if ( conf->bootstrap )
		conf->bootstrap->prev = bs;

	bs->prev = NULL;
	bs->next = conf->bootstrap;
	conf->bootstrap = bs;
}

void lc6config_free(LC6_CTX *conf) {
	LC6_BOOTSTRAP *bnode = conf->bootstrap;
	LC6_USER      *unode = conf->user;
	LC6_FRIEND      *fnode = conf->friend;

	while(bnode) {
		LC6_BOOTSTRAP *next = bnode->next;
		free(bnode);
		bnode=next;
	}

	while(unode) {
		LC6_USER *next = unode->next;
		free(unode);
		unode=next;
	}
	while(fnode) {
		LC6_FRIEND *next = fnode->next;
		free(fnode);
		fnode=next;
	}

	if ( conf->node )
		free(conf->node);

	free(conf);
}

int lc6config_save(LC6_CTX *conf, unsigned char *password) {

	LC6_BOOTSTRAP *bootstrap_save = NULL;
	LC6_BOOTSTRAP *bootstrap_node = conf->bootstrap;
	int bootstrap_len = 0;

	LC6_USER *user_save = NULL;
	LC6_USER *user_node = conf->user;
	int user_len = 0;

	LC6_FRIEND *friend_save = NULL;
	LC6_FRIEND *friend_node = conf->friend;
	int friend_len = 0;

	int i;
	char file[MAXPATHLEN];

	while(bootstrap_node) {
		bootstrap_len++;
		bootstrap_node = bootstrap_node->next;
	}
	bootstrap_node = conf->bootstrap;

	bootstrap_save = malloc(sizeof(LC6_BOOTSTRAP)*bootstrap_len);
	assert(bootstrap_save);
	memset(bootstrap_save, '\0', sizeof(LC6_BOOTSTRAP)*bootstrap_len);

	for(i=0; i<bootstrap_len; i++) {
		memcpy(bootstrap_save + i, bootstrap_node, sizeof(LC6_BOOTSTRAP));
		(bootstrap_save+i)->prev = NULL;
		(bootstrap_save+i)->next = NULL;
		bootstrap_node = bootstrap_node->next;
	}
	snprintf(file, sizeof(file), "%s/%s", conf->path, LC6_CONFIG_BOOTSTRAP);
	lc6crypto_writefile(file, password, (unsigned char*)bootstrap_save, bootstrap_len * sizeof(LC6_BOOTSTRAP));

	free(bootstrap_save);

	while(user_node) {
		user_len++;
		user_node = user_node->next;
	}

	user_save = malloc(sizeof(LC6_USER)*user_len);
	assert(user_save);
	memset(user_save, 0, sizeof(LC6_USER)*user_len);

	for(i=0; i<user_len; i++) {
		LC6_USER *node = conf->user + i;
		memcpy(user_save + i, node, sizeof(LC6_USER));
		(user_save+i)->prev = NULL;
		(user_save+i)->next = NULL;
	}

	snprintf(file, sizeof(file), "%s/%s", conf->path, LC6_CONFIG_USER);
	lc6crypto_writefile(file, password, (unsigned char*)user_save, user_len * sizeof(LC6_USER));

	free(user_save);

	while(friend_node) {
		friend_len++;
		friend_node = friend_node->next;
	}

	friend_save = malloc(sizeof(LC6_FRIEND)*friend_len);
	assert(friend_save);
	memset(friend_save, 0, sizeof(LC6_FRIEND)*friend_len);

	for(i=0; i<friend_len; i++) {
		LC6_FRIEND *node = conf->friend + i;
		memcpy(friend_save + i, node, sizeof(LC6_FRIEND));
		(friend_save+i)->prev = NULL;
		(friend_save+i)->next = NULL;
	}

	snprintf(file, sizeof(file), "%s/%s", conf->path, LC6_CONFIG_FRIEND);
	lc6crypto_writefile(file, password, (unsigned char*)friend_save, friend_len * sizeof(LC6_FRIEND));

	free(friend_save);

	return 1;
}
